<?php

require_once 'base/FpagozoferiasEntityBase.php';


/**
 * FpaGozoFeriasEntity
 *
 * @package Entity
 *
 * @author  luistavares
 *
 */
class FpagozoferiasEntity extends FpagozoferiasEntityBase
{

    private $_mes_1a_parte;
    private $_dias_1a_parte;
    private $_mes_2a_parte;
    private $_dias_2a_parte;
    private $_dias_abono;
    private $_dtInicioGozo;
    private $_dtFimGozo;
    private $_nr_ferias_preparado;


    /**
     * Construtor.
     *
     * @param string $id Id.
     *
     * @author luistavares - 20/06/2011 - Adequa??o do m?todo para o padr?o do CodeSniffer.
     */
    public function __construct($id = null){
        parent::__construct($id);
        $this->_mes_1a_parte = new TkDateTime();
        $this->_mes_2a_parte = new TkDateTime();
        $this->_dtInicioGozo = new TkDateTime();
        $this->_dtFimGozo = new TkDateTime();
    }


    /**
     * Retorna $nr_ferias_preparado.
     *
     * @return integer
     *
     * @author luistavares - 20/06/2011 - Adequa??o do m?todo para o padr?o do CodeSniffer.
     */
    public function getNrFeriasPreparado(){
        if ($this->_nr_ferias_preparado > 0) {
            return $this->_nr_ferias_preparado;
        } else {
            return null;
        }
    }


    /**
     * Retorna $mes_1a_parte.
     *
     * @return TkDateTime
     *
     * @author luistavares - 20/06/2011 - Adequa??o do m?todo para o padr?o do CodeSniffer.
     */
    public function getMes1aParte(){
        return $this->_mes_1a_parte;
    }


    /**
     * Retorna $dias_1a_parte.
     *
     * @return number
     *
     * @author luistavares - 20/06/2011 - Adequa??o do m?todo para o padr?o do CodeSniffer.
     */
    public function getDias1aParte(){
        return $this->_dias_1a_parte;
    }


    /**
     * Retorna $mes_2a_parte.
     *
     * @return TkDateTime
     *
     * @author luistavares - 20/06/2011 - Adequa??o do m?todo para o padr?o do CodeSniffer.
     */
    public function getMes2aParte(){
        return $this->_mes_2a_parte;
    }


    /**
     * Retorna $dias_2a_parte.
     *
     * @return number
     *
     * @author luistavares - 20/06/2011 - Adequa??o do m?todo para o padr?o do CodeSniffer.
     */
    public function getDias2aParte(){
        return $this->_dias_2a_parte;
    }


    /**
     * Retorna $dias_abono.
     *
     * @return number
     *
     * @author luistavares - 20/06/2011 - Adequa??o do m?todo para o padr?o do CodeSniffer.
     */
    public function getDiasAbono(){
        return $this->_dias_abono;
    }


    /**
     * Retorna data inicio de gozo.
     *
     * @return string
     */
    public function getDtInicioGozo(){
        return $this->_dtInicioGozo;
    }


    /**
     * Retorna data fim de gozo.
     *
     * @return string
     */
    public function getDtFimGozo(){
        return $this->_dtFimGozo;
    }


    /**
     * Retorna Gozo Ferias por Nr. V?nculo.
     *
     * @param integer $nr_org     N?mero de Organiza??o.
     * @param integer $nr_vinculo N?mero de V?nculo.
     *
     * @return void
     *
     * @author alessandrobartels -
     * 		   luistavares - 20/06/2011 - Adequa??o do m?todo para o padr?o do CodeSniffer.
     *         alessandrobartels - 28/06/2011 - Modificado a condicao. Antes comparava se o retornaItem retornava
     *         									um objeto. O retorno eh um array.
     */
    public function retornaGozoFerias($nr_org, $nr_vinculo){

        $this->resetItems();
        $this->setNrorg($nr_org);
        $resultado = Fpagozoferias::retornaItem($nr_org, $nr_vinculo);
        if (count($resultado) > 0) {
            foreach ($resultado as $entity_obj) {
                $this->addItem($entity_obj, $entity_obj->getNrgozoferias());
            }

            /*return $this->getItems();*/
        }
    }


    /**
     * Procura pelo gozo de ferias pela data de inicio parametrizada.
     *
     * @param TkDateTime $dataInicioPeriodo Data de inicio de periodo do gozo de ferias.
     *
     * @return FpagozoferiasEntity
     *
     * @author alessandrobartels 05/08/2011
     */
    public function procuraGozoFerias(TkDateTime $dataInicioPeriodo){
        foreach ($this->getItems() as $item) {
            if ($dataInicioPeriodo->equals($item->getDtinigozoferias(), TkDateTime::FORMAT_DATE) === true) {
                return $item;
            }
        }
    }


    /**
     * Retorna per?odo de f?rias entre duas datas (retorna em dias).
     *
     * @param string $data_ini Data Inicio.
     * @param string $data_fim Data Fim.
     *
     * @return integer
     *
     * @author luistavares - 16/06/2011
     */
    public function retornaPeriodoFerias ($data_ini, $data_fim){
        $aux = new TkDateTime($data_fim, TkDateTime::FORMAT_DATE);
        $aux2 = new TkDateTime($data_ini, TkDateTime::FORMAT_DATE);
        $periodo = $aux2->daysAfter($aux);
        return $periodo;
    }


    /**
     * Inicializa o array de ferias.
     *
     * @param array   $ferias      Array de ferias.
     * @param integer $diasPeriodo Dias do periodo de ferias.
     *
     * @return array
     *
     * @author alessandrobartels - 22/06/2011 - Criado para adequar ao padr?o Codesniffer.
     */
    private function _inicializaArrayFerias(array $ferias, $diasPeriodo){
        /* alessandrobartels 26/07/2011 - retirado comparacao $i <= */
        for ($i=0; $i <= $diasPeriodo; $i++) {
            $ferias[$i] = -1;
        }

        return $ferias;
    }


    /**
     * Inclui registro de gozo de ferias.
     *
     * @param integer $nrOrg           Numero da Organizacao de Trabalho.
     * @param integer $nrFerias        Numero identificador de ferias.
     * @param string  $dtIniGozo       Data de inicio de gozo de ferias.
     * @param string  $dtFimGozo       Data fim de gozo de ferias.
     * @param integer $qtdDiasAbono    Quantidade de dias de abono ("vender dias").
     * @param string  $dtRetornoFerias Data de retorno das ferias.
     * @param string  $idAdiant13Sal   Identificador de adiantamento de 13 salario nas ferias.
     *
     * @throws Exception $e Propaga excecao - Erro na insercao do registro.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function incluiGozoFerias(
        $nrOrg, $nrFerias, $dtIniGozo, $dtFimGozo, $qtdDiasAbono, $dtRetornoFerias, $idAdiant13Sal
    ){
        try{
            $gozoFerias = new self();

            $gozoFerias->setNrorg($nrOrg);
            $gozoFerias->setNrferias($nrFerias);
            $gozoFerias->setDtinigozoferias($dtIniGozo);
            $gozoFerias->setDtfimgozoferias($dtFimGozo);
            $gozoFerias->setQtdiasabonoferi($qtdDiasAbono);
            $gozoFerias->setDtretornoferias($dtRetornoFerias);
            $gozoFerias->setIdadiant13salar($idAdiant13Sal);

            $gozoFerias->persist();

            unset($gozoFerias);

        }catch(Exception $e){
            throw $e;
        }
    }


    /**
     * Retorna Dias F?rias.
     *
     * @param string $data_ini_per Data In?cio Per?odo.
     * @param string $data_fim_per Data Fim Per?odo.
     *
     * @return array
     *
     * @author luistavares - 20/06/2011 - Adequa??o do m?todo para o padr?o do CodeSniffer.
     * 		   alessandrobartels - 22/06/2011 - Retirado la?o de inicializa??o do array e passado para um novo m?todo.
     */
    public function retornaDiasFerias($data_ini_per, $data_fim_per){
        $ferias = array();

        $date_aux = new TkDateTime($data_fim_per, TkDateTime::FORMAT_DATE);
        $dias_periodo = $data_ini_per->daysAfter($date_aux);

        $ferias = $this->_inicializaArrayFerias($ferias, $dias_periodo);
        $data_ini_aux = new TkDateTime();
        $data_fim_aux = new TkDateTime();


        $itens = $this->getItems();
        if (count($itens) > 0){
            foreach ($itens as $item) {
                if ($item->getDtinigozoferias() !== false) {
                    $data_ini_aux->set($item->getDtinigozoferias());
                    if ($data_ini_aux->isLater($data_fim_per) === true) {
                        // Afastamento comecou depois do periodo tratado.
                        break;
                    }

                    try{
                        $data_fim_aux->set($item->getDtfimgozoferias(), TkDateTime::FORMAT_DATE);
                    }catch (Exception $e){
                        $data_fim_aux = null;
                    }

                    if (($data_fim_aux === null)
                        || ($data_fim_aux->isLater($data_fim_per) === true)
                    ) {
                        // Comecou e acabou depois ou sem data fim de gozo.

                        if($data_fim_aux === null){
                            $data_fim_aux = new TkDateTime();
                        }

                        $data_fim_aux->setDate($data_fim_per);
                        $this->_dtInicioGozo->set($data_ini_aux, TkDateTime::FORMAT_DATE);
                        $this->_dtFimGozo->set($data_fim_aux, TkDateTime::FORMAT_DATE);
                    }
                    //tk_warning($data_ini_aux->toString('dd/MM/YYYY').'<-I.Aux  I->'.$data_ini_per);
                    //tk_warning($data_fim_aux->toString('dd/MM/YYYY').'<-F.Aux  F->'.$data_fim_per);

                    if ((($data_ini_aux->isLater($data_ini_per) === true)
                        ||($data_ini_aux->equals($data_ini_per) === true))
                        &&(($data_ini_aux->isEarlier($data_fim_per) === true)
                        ||($data_ini_aux->equals($data_fim_per) === true))
                    ) {
                        // Comecou e acabou durante.
                        // Conta dias de data_ini_aux ate data_fim_aux.
                        $this->_dtInicioGozo->set($data_ini_aux, TkDateTime::FORMAT_DATE);
                        $this->_dtFimGozo->set($data_fim_aux, TkDateTime::FORMAT_DATE);
                        $dia_ini = $data_ini_per->daysAfter($data_ini_aux) - 1;
                        // Ajustado para 0 quando negativo pois o array começa de 0 e nos demais casos trabalha com a posicao anterior;
                        if ($dia_ini < 0){
                            $dia_ini = 0;
                        }
                        // BORBA 30/09/2013 Retirado o simbolo =, o array era montado com 1 dia amais de férias
                        //$qtd_dias = $data_ini_aux->daysAfter($data_fim_aux);
                        // BORBA 01/04/2015 antigo não funciona quando a $data_ini_aux é igual ao $data_fim_aux // OK.
						if($data_ini_aux->daysAfter($data_fim_aux)==0){
						 $qtd_dias = 1;
						} else{
						 $qtd_dias = $data_ini_aux->daysAfter($data_fim_aux); // OK.
						}
						for ($i=0; $i < $qtd_dias; $i++) {
                            $ferias[$i+$dia_ini] = 1;
                        }
                        //tk_warning('<pre>'.print_r($ferias, true).'</pre>');
                    } elseif (($data_ini_aux->isEarlier($data_ini_per))
                        && (($data_fim_aux->isEarlier($data_fim_per)) ||  ($data_fim_aux->equals($data_fim_per)))
                    ) {
                        if (($data_fim_aux->isLater($data_ini_per) === true)
                            || ($data_fim_aux->equals($data_ini_per) === true)
                        ) {
                            // Comecou antes e acabou durante.
                            // Conta dias do data_ini_per ate data_fim_aux.
                            $this->_dtInicioGozo->set($data_ini_aux, TkDateTime::FORMAT_DATE);
                            $this->_dtFimGozo->set($data_fim_aux, TkDateTime::FORMAT_DATE);
                            //$qtd_dias = $data_ini_per->daysAfter($data_fim_aux);
							// BORBA 01/04/2015 antigo não funciona quando a $data_ini_per é igual ao $data_fim_aux // OK.
							if($data_ini_per->daysAfter($data_fim_aux)==0){
							 $qtd_dias = 1;
							} else{
							 $qtd_dias = $data_ini_per->daysAfter($data_fim_aux); // OK.
							}
                            for ($i=0; $i < $qtd_dias; $i++) { // Retirado o simbolo =, o array era montado com 1 dia amais de férias BORBA 22/04/2013
                                $ferias[$i] = 1;
                            }
                        } else {
                            // Comecou antes e terminou antes.
                            true;
                        }
                    }
                }
            }
        } else {
           $this->_dtInicioGozo->set(TkDateTime::DUMMY_DATE);
           $this->_dtFimGozo->set(TkDateTime::DUMMY_DATE);
        }
        return $ferias;
    }


    /**
     * Tratamento do Gozo de Ferias.
     *
     * @param \TkDateTime $inicio_periodo_saida In?cio do Per?odo de Sa?da.
     * @param \TkDateTime $fim_periodo_saida    Fim  do Per?odo de Sa?da.
     * @param \TkDateTime $mes_competencia      M?s de Compet?ncia.
     *
     * @return void
     *
     * @author alessandrobartels
     *         luistavares - 20/06/2011 - Adequação do metodo para o padrão do CodeSniffer.
     */
    public function trataGozoFerias(
        \TkDateTime $inicio_periodo_saida, \TkDateTime $fim_periodo_saida, \TkDateTime $mes_competencia
    ){
        $pri_dia_mes = new \TkDateTime($mes_competencia, TkDateTime::FORMAT_DATE);
        $pri_dia_mes->setDay(1);

        $ult_dia_mes = new \TkDateTime($mes_competencia, TkDateTime::FORMAT_DATE);
        $ult_dia_mes->setDay(1)->addMonth(1)->subDay(1);

        $pri_dia_mes_seg = new \TkDateTime($mes_competencia, TkDateTime::FORMAT_DATE);
        $pri_dia_mes_seg->addMonth(1)->setDay(1);

        $ult_dia_mes_seg = new \TkDateTime($mes_competencia, TkDateTime::FORMAT_DATE);
        $ult_dia_mes_seg->setDay(1)->addMonth(2)->subDay(1);

        $pri_dia_mes_seg_seg = new \TkDateTime($ult_dia_mes_seg, TkDateTime::FORMAT_DATE);
        $pri_dia_mes_seg_seg->addMonth(1)->setDay(1);

        $data_ini_aux = new \TkDateTime();
        $data_fim_aux = new \TkDateTime();

        /**  @author thalesvalias 16/11/2012 - inicializacao adicionada para resolver o problema de que quando um vinculo
        possui gozo e outro apos esse nao possui o nr_ferias_preparado continua com o valor do vinculo anterior, acarretando
        em erro.*/
        $this->_nr_ferias_preparado = null;
        foreach ($this->getItems() as $item) {
            $this->_dtInicioGozo->set($item->getDtinigozoferias(), \TkDateTime::FORMAT_DATE);
            $this->_dtFimGozo->set($item->getDtfimgozoferias(), \TkDateTime::FORMAT_DATE);

            $data_ini_aux->set($item->getDtinigozoferias(), \TkDateTime::FORMAT_DATE);
            if (( $data_ini_aux->isLater($inicio_periodo_saida) === true
                || $data_ini_aux->equals($inicio_periodo_saida) === true)
                && ($data_ini_aux->isEarlier($fim_periodo_saida) === true
                    || $data_ini_aux->equals($fim_periodo_saida) === true)
            ) {
                // Inicio de Gozo de Ferias esta dentro do periodo tratado.
                $this->_nr_ferias_preparado = $item->getNrFerias();

                $data_fim_aux->set($item->getDtfimgozoferias(), \TkDateTime::FORMAT_DATE);

                if ($data_ini_aux->isEarlier($pri_dia_mes) === true) {
                    /* Se data de inicio de gozo de ferias for anterior ao mes de */
                    /* preparacao, gerar apenas verba de ferias no mes seguinte */
                    /* complementando ferias */
                    /*	ls_inic_gozo = C_PREP_DEPOIS_GOZO*/
                    $this->_mes_1a_parte->set($mes_competencia);
                    $this->_dias_1a_parte = 0;
                    $this->_mes_2a_parte->set($mes_competencia);

                    // BORBA 14/05/2013 antigo não funciona quando a $data_fim_aux é igual ao $pri_dia_mes - $this->_dias_2a_parte = ($pri_dia_mes->daysAfter($data_fim_aux));	// Ok.
                    if($pri_dia_mes->daysAfter($data_fim_aux)==0){
                        $this->_dias_2a_parte = 1;
                    } else{
                        $this->_dias_2a_parte = $pri_dia_mes->daysAfter($data_fim_aux);  // OK.
                    }

                    $this->_dias_abono = 0;
                } else {
                    $this->_dias_abono = $item->getQtdiasabonoferi(); /*qtdiasabonoferi;*/
                    if ($data_ini_aux->isEarlier($ult_dia_mes) === true
                        || $data_ini_aux->equals($ult_dia_mes) === true
                    ) {
                        /* Se data de in?cio de gozo de f?rias estiver dentro do
                         * m?s de prepara??o
                         * ls_inic_gozo = C_PREP_DENTRO_GOZO
                         */
                        $this->_mes_1a_parte->set($mes_competencia);
                        if ($data_fim_aux->isLater($ult_dia_mes) === true) {

                            // BORBA 01/07/2014 antigo não funciona quando a $data_ini_aux é igual ao $ult_dia_mes - $this->_dias_1a_parte = $data_ini_aux->daysAfter($ult_dia_mes);
                            if($data_ini_aux->daysAfter($ult_dia_mes)==0){
                                $this->_dias_1a_parte = 1;
                            }else{
                                $this->_dias_1a_parte = $data_ini_aux->daysAfter($ult_dia_mes);  // OK.
                            }

                            $this->_mes_2a_parte->set($pri_dia_mes_seg);

                            // BORBA 14/05/2013 antigo não funciona quando a $data_fim_aux é igual ao $pri_dia_mes_seg - $this->_dias_2a_parte = $pri_dia_mes_seg->daysAfter($data_fim_aux);
                            if($pri_dia_mes_seg->daysAfter($data_fim_aux)==0){
                                $this->_dias_2a_parte = 1;
                            } else{
                                $this->_dias_2a_parte = $pri_dia_mes_seg->daysAfter($data_fim_aux);  // OK.
                            }

                        } else {
                            $this->_dias_1a_parte = $data_ini_aux->daysAfter($data_fim_aux);    // OK.
                            $this->_mes_2a_parte->set($mes_competencia);
                            $this->_dias_2a_parte = 0;
                        }
                    } else {
                        /*	ls_inic_gozo = C_PREP_ANTES_GOZO*/
                        $this->_mes_1a_parte->set($pri_dia_mes_seg);
                        if ($data_fim_aux->isLater($ult_dia_mes_seg) === true) {
                            $this->_dias_1a_parte = $data_ini_aux->daysAfter($ult_dia_mes_seg);   // OK.
                            $this->_mes_2a_parte->set($pri_dia_mes_seg_seg);

                            // BORBA 14/05/2013 antigo não funciona quando a $data_fim_aux é igual ao $pri_dia_mes_seg_seg - $this->_dias_2a_parte = $pri_dia_mes_seg_seg->daysAfter($data_fim_aux);   // OK.
                            if($pri_dia_mes_seg_seg->daysAfter($data_fim_aux)==0){
                                $this->_dias_2a_parte = 1;
                            } else{
                                $this->_dias_2a_parte = $pri_dia_mes_seg_seg->daysAfter($data_fim_aux);  // OK.
                            }

                        } else {
                            $this->_dias_1a_parte = $data_ini_aux->daysAfter($data_fim_aux);     // OK.
                            $this->_mes_2a_parte->set($pri_dia_mes_seg);
                            $this->_dias_2a_parte = 0;
                        }
                    }
                }
            }
        }

        /*		return $ferias;*/
    }


    /**
     * Retorna lista de Objetos de Gozo de Ferias.
     *
     * @param integer $nrferias N?mero das ferias.
     * @param integer $nrorg    N?mero da Organiza??o.
     *
     * @return array FpagozoferiasEntity
     *
     * @author alessandrobartels
     */
    public function retornaItemPorNrFerias($nrferias, $nrorg){
        return Fpagozoferias::retornaItemPorNrFerias($nrferias, $nrorg);
    }

    public function insereGozoFerias($nrorg, $nrferias, $dtinigozo, $dtfimgozo, $qtddiasabono, $dtretornoferias,
                                     $idadiant13sal)
    {

        $this->setNrorg($nrorg);
        $this->setNrferias($nrferias);
        $this->setDtinigozoferias($dtinigozo);
        $this->setDtfimgozoferias($dtfimgozo);
        $this->setQtdiasabonoferi($qtddiasabono);
        $this->setDtretornoferias($dtretornoferias);
        $this->setIdadiant13salar($idadiant13sal);
    }

    public function getDiasDataInicioAteDataFim() {
        $dtInicio  = new \TkDateTime($this->getDtinigozoferias(), \TkDateTime::FORMAT_DATE);
        $dtFim     = new \TkDateTime($this->getDtfimgozoferias(), \TkDateTime::FORMAT_DATE);
        $diferenca = $dtInicio->daysUntil($dtFim);
        return $diferenca;
    }

    //05/03/2015 BORBA - Retorna dias de gozo caso exista um gozo anterior para o mesmo período
    public function getDiasGozados() {
        $dtInicio  = new \TkDateTime($this->getDtinigozoferias(), \TkDateTime::FORMAT_DATE);
        $dtFim     = new \TkDateTime($this->getDtfimgozoferias(), \TkDateTime::FORMAT_DATE);
        $diferenca = $dtInicio->daysAfter($dtFim);
        return $diferenca;
    }

}

